#
# ir.context - Infrared symbol lookup contexts
#
# Context simply collects objects useful for talking about IR things.
#
# Copyright 2011-2016 Perry The Cynic. All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#	http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
import ir
import ir.db


class Context(object):

	def __init__(self, database=None):
		self.db = database
		self.group = None

	def set_db(self, db):
		if self.db:
			self.db.close()
		self.db = db

	def to_string(self, symbol):
		if symbol is None:
			return None
		if self.db:
			match = self.db.match_code(symbol.code())
			if match:
				return "%s/%s" % (match['grouping'], match['name'])
		return str(symbol)

	def from_string(self, form):
		if form is None:
			return None
		if form[0] in "<0123456789":	# <form:content> or raw hex
			return ir.Symbol.from_text(form)
		if self.db:						# try db symbol
			return self.db.find_symbol(form)
		return None						# nope
