#
# Global Cache interface plugin for Indigo 5+
#
# Copyright 2011-2016 Perry The Cynic. All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#	http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
import socket

import ir.gcache

import cyin
import cyin.devstate
from cyin import debug, error
from cyin.check import *


#
# Device objects
#
class GCNet(cyin.devstate.IPDevice, cyin.devstate.Idler):
	""" A device representing a GC network device itself.

		This device represents the actual network-addressable hardware platform.
		This is what we connect to, and it's the target of our device discovery.
	"""
	config_version = 1		# new states

	active_learner = None

	@property
	def gcnet(self): return self.target

	DEFAULT_PORT = ir.gcache.GCNET_PORT

	def connected(self, socket):
		self.target = ir.gcache.GCNet(cyin.plugin, callout=self._event)
		self.proceed("exploring")
		self.gcnet.open(socket)
		self.idle_update()
	
	def stop(self):
		self.idle_cancel()
		super(GCNet, self).stop()

	def _event(self, ctx, data=None):
		if ctx.error:
			return self.fail_soft(ctx)
		if ctx.state == 'ready':
			self.proceed("ready", log="ready with %d device(s)" % len(data), recovered=True)
			debug(self.name, "version", self.gcnet.version)
		elif ctx.state == 'learning':
			debug("%s learning ir" % ("start" if data else "stop"))
		elif ctx.state == 'learned':
			self.gcnet.stop_learning()
			if self.active_learner:
				reply = self.active_learner
				self.active_learner = None
				code = data.classify()
				reply.callout('learned', code)
		elif ctx.state == 'END':
			self.reconnect()
		elif ctx.state == 'close':
			pass
		else:
			error("unexpected event", ctx, data)

	def learn_symbol(self, reply):
		self.active_learner = reply	# who to send our reply to
		if reply:
			self.gcnet.start_learning()
		else:
			self.gcnet.stop_learning()

	@classmethod
	def all_devices(cls, devclass=None):
		for net in GCNet.all():
			if net.ready():
				for gcdev in net.gcnet.all_type(devclass):
					yield (net, gcdev.address)

	@classmethod
	def find_host(cls, hostname):
		for dev in cls.all():
			if dev.gcnet.hostname == hostname:
				return dev
